-- $Id: timed_effects.lua 1202 2015-01-27 00:54:34Z dsb $
-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham ("Zizzo")
-- dsb-tome@killerbunnies.org

local Entity = require 'engine.Entity'

newEffect {
  name = 'GOING_TO_LANDMARK',
  desc = 'Going to Landmark',
  image = 'effects/going_to_landmark.png',
  display_entity = Entity.new {
    image = 'effects/going_to_landmark.png',
    is_effect = true,
  },
  long_desc = function(self, eff)
    local key = self:callEffect(eff.effect_id, 'get_key_binds', "' or '")
    return ([[You are traveling to the landmark '%s at %d,%d'.  The auto-explore function (currently bound to '%s') will continue to this landmark in preference to other destinations, until you arrive at the landmark, leave the level, or cancel this effect.]]):format(eff.target_name, eff.target_x, eff.target_y, key)
  end,
  type = 'other',
  subtype = { landmark=true },
  no_stop_enter_worlmap = true,
  no_stop_resting = true,
  status = 'beneficial',
  decrease = 0,
  cancel_on_level_change = true,
  on_timeout = function(self, eff)
    local tx, ty = eff.target_x, eff.target_y
    if self.x >= tx - 1 and self.x <= tx + 1 and self.y >= ty - 1 and self.y <= ty + 1
    then
      self:removeEffect(self.EFF_GOING_TO_LANDMARK, true, true)
    end
  end,
  -- The following two methods are borrowed from my T2 module, to create a
  -- formmated description of key(s) bound to a KeyBind virtual action.
  get_key_string = function(who, p, ks)
    -- Slightly modified from engine.KeyBind:formatKeyString() for better
    -- readability (f.i. Ctrl-Q instead of CQ).
    if not ks then return '<<?? unknown>>' end

    local function mods(sfx, ctrl, shift, alt, meta)
      return (meta == 'true' and 'Meta-' or '') ..
	     (alt == 'true' and 'Alt-' or '') ..
	     (shift == 'true' and 'Shift-' or '') ..
	     (ctrl == 'true' and 'Ctrl-' or '') .. sfx
    end

    if ks:find("^uni:") then
      return ks:sub(5)
    elseif ks:find("^sym:") then
      local i, j, sym, ctrl, shift, alt, meta = ks:find("^sym:([^:]+):([a-z]+):([a-z]+):([a-z]+):([a-z]+)$")
      if not i then return '<<?? unknown>>' end
      if sym:sub(1, 1) == "=" then
	sym = sym:sub(2)
      else
	local KeyBind = require('engine.KeyBind')
	if tonumber(sym) then sym = tonumber(sym) else sym = KeyBind[sym] end
	sym = core.key.symName(sym)
	-- Slight Hack(TM):  symName('_z') returns 'Z'.  We want 'z' with
	-- no modifiers, 'Shift-z' with just shift, and eg. 'Ctrl-Z' with
	-- anything else.
	if string.len(sym) == 1 and ctrl ~= 'true' and alt ~= 'true' and meta ~= 'true' then
	  sym = string.lower(sym)
	end
      end
      return mods(sym, ctrl, shift, alt, meta)
    elseif ks:find("^mouse:") then
      local i, j, sym, ctrl, shift, alt, meta = ks:find("^mouse:([a-zA-Z0-9]+):([a-z]+):([a-z]+):([a-z]+):([a-z]+)$")
      if not i then return '<<?? unknown>>' end
      return mods(sym, ctr, shift, alt, meta)
    elseif ks:find("^gest:") then
      local i, j, sym = ks:find("^gest:([a-zA-Z0-9]+)$")
      if not i then return '<<?? unknown>>' end
      return sym
    else
      return '<<?? unknown>>'
    end
  end,
  get_key_binds = function(who, p, sep)
    local KeyBind = require('engine.KeyBind')
    if not KeyBind.binds_def.RUN_AUTO then return '<<?? unknown>>' end
    local t = KeyBind:getBindTable(KeyBind.binds_def.RUN_AUTO)
    local ret = {}
    for i = 1, 3 do
      if t[i] and #ret < 2 then
	ret[#ret+1] = who:callEffect(p.effect_id, 'get_key_string', t[i])
      end
    end
    if #ret == 0 then return '<<?? unknown>>' end
    return sep and table.concat(ret, sep) or ret[1]
  end,
}
